Attribute VB_Name = "MEnsureVisible"
  Option Explicit
  ' demo project showing how to use the Windows API to make sure a
  ' given control on a MDI child form is visible whthin the MDI parent
  ' by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at your own
  ' risk.

  Public Type RECT
    Left As Long
    Top As Long
    Right As Long
    Bottom As Long
  End Type

  Public Const GW_CHILD As Long = 5&
  Public Const SM_CXVSCROLL As Long = 2&
  Public Const SM_CYHSCROLL As Long = 3&
  
  Public Const SW_SCROLLCHILDREN As Long = &H1&
  Public Const SW_INVALIDATE As Long = &H2&
  Public Const SW_ERASE As Long = &H4&

  Public Const SB_THUMBPOSITION  As Long = 4&
  Public Const SB_THUMBTRACK  As Long = 5&
  Public Const SB_ENDSCROLL  As Long = 8&

  Private Enum SCROLL_MSG
    WM_HSCROLL = &H114&
    WM_VSCROLL = &H115&
  End Enum


  Public Declare Function GetWindowRect Lib "user32" (ByVal hWnd&, lpRect As RECT) As Long
  Public Declare Function PtInRect Lib "user32" (lpRect As RECT, ByVal ptx&, ByVal pty&) As Long
  Public Declare Function GetWindow Lib "user32" (ByVal hWnd&, ByVal wCmd&) As Long
  Public Declare Function GetSystemMetrics Lib "user32" (ByVal nIndex&) As Long
  Public Declare Function ScrollWindowEx Lib "user32" (ByVal hWnd&, ByVal dx&, ByVal dy&, ByVal lprcScroll&, ByVal lprcClip&, ByVal hrgnUpdate&, ByVal lprcUpdate&, ByVal fuScroll&) As Long
  Public Declare Function SendMessage Lib "user32" Alias "SendMessageA" (ByVal hWnd&, ByVal wMsg&, wParam As Any, lParam As Any) As Long

Public Sub EnsureVisible(oCtl As Control, oMDIfrm As MDIForm)

  Dim hMDIClient&, rcControl As RECT, rcMDIClient As RECT, bNotCompletelyVisible As Boolean
  
  ' get the hWnd of the client area of the MDI parent
  hMDIClient = GetWindow(oMDIfrm.hWnd, GW_CHILD)
  
  ' get the rect (in screen coordinates) of the passed control and the MDI client area
  Call GetWindowRect(oCtl.hWnd, rcControl)
  Call GetWindowRect(hMDIClient, rcMDIClient)
  
  ' deduct the width and height of a scrollbar from the MDI client area rect.  also
  ' deduct a little padding so that the focused control wont be right up against the
  ' edge of the client area....
  With rcMDIClient
    .Left = .Left + 10
    .Top = .Top + 10
    .Right = .Right - (GetSystemMetrics(SM_CXVSCROLL) + 10)
    .Bottom = .Bottom - (GetSystemMetrics(SM_CYHSCROLL) + 10)
  End With
  
  
  With rcControl
    ' determine whether or not the control is currently visible.  if it isn't, set the flag
    Select Case 0
      Case PtInRect(rcMDIClient, .Left, .Top): bNotCompletelyVisible = True
      Case PtInRect(rcMDIClient, .Right, .Top): bNotCompletelyVisible = True
      Case PtInRect(rcMDIClient, .Left, .Bottom): bNotCompletelyVisible = True
      Case PtInRect(rcMDIClient, .Right, .Bottom): bNotCompletelyVisible = True
    End Select
    
    
    ' if the control isn't visible, make it so!
    ' scroll the client area of the MDI parent and then make the scrollbars match the current position
    If bNotCompletelyVisible Then
      If rcMDIClient.Left > .Left Then
        Call ScrollWindowEx(hMDIClient, rcMDIClient.Left - .Left, 0&, 0&, 0&, 0&, 0&, SW_ERASE Or SW_INVALIDATE Or SW_SCROLLCHILDREN)
        
        MoveScrollbar hMDIClient, WM_HSCROLL
      End If
      
      If rcMDIClient.Top > .Top Then
        Call ScrollWindowEx(hMDIClient, 0&, rcMDIClient.Top - .Top, 0&, 0&, 0&, 0&, SW_ERASE Or SW_INVALIDATE Or SW_SCROLLCHILDREN)

        MoveScrollbar hMDIClient, WM_VSCROLL
      End If
      
      If rcMDIClient.Right < .Right Then
        Call ScrollWindowEx(hMDIClient, -(.Right - rcMDIClient.Right), 0&, 0&, 0&, 0&, 0&, SW_ERASE Or SW_INVALIDATE Or SW_SCROLLCHILDREN)

        MoveScrollbar hMDIClient, WM_HSCROLL
      End If
      
      If rcMDIClient.Bottom < .Bottom Then
        Call ScrollWindowEx(hMDIClient, 0&, -(.Bottom - rcMDIClient.Bottom), 0&, 0&, 0&, 0&, SW_ERASE Or SW_INVALIDATE Or SW_SCROLLCHILDREN)
    
        MoveScrollbar hMDIClient, WM_VSCROLL
      End If
    End If
  End With

End Sub

Private Sub MoveScrollbar(ByVal hWnd&, ByVal eMessage As SCROLL_MSG)

  ' cause either the virtical or horizontal scrollbar to move to the
  ' current correct position for the MDI client DC
  
  Call SendMessage(hWnd, eMessage, ByVal SB_THUMBTRACK, ByVal 0&)
  Call SendMessage(hWnd, eMessage, ByVal SB_THUMBPOSITION, ByVal 0&)
  Call SendMessage(hWnd, eMessage, ByVal SB_ENDSCROLL, ByVal 0&)
  
  Call SendMessage(hWnd, eMessage, ByVal SB_THUMBTRACK, ByVal 0&)
  Call SendMessage(hWnd, eMessage, ByVal SB_THUMBPOSITION, ByVal 0&)
  Call SendMessage(hWnd, eMessage, ByVal SB_ENDSCROLL, ByVal 0&)

End Sub
