Attribute VB_Name = "MTimezoneInfo"
  Option Explicit
  ' demo project showing how to call the GetTimeZoneInformation API
  ' function by Bryan Stafford of New Vision Software - newvision@mvps.org
  ' this demo is released into the public domain "as is" without
  ' warranty or guaranty of any kind.  In other words, use at
  ' your own risk.
  

  Public Type SYSTEMTIME
    wYear As Integer
    wMonth As Integer
    wDayOfWeek As Integer
    wDay As Integer
    wHour As Integer
    wMinute As Integer
    wSecond As Integer
    wMilliseconds As Integer
  End Type

  Public Type TIME_ZONE_INFORMATION
    Bias As Long
    StandardName(63) As Byte  ' <-- zero based array so there are actually 64 bytes in the array
    StandardDate As SYSTEMTIME
    StandardBias As Long
    DaylightName(63) As Byte  ' <-- zero based array so there are actually 64 bytes in the array
    DaylightDate As SYSTEMTIME
    DaylightBias As Long
  End Type

  Public Const TIME_ZONE_ID_INVALID As Long = &HFFFFFFFF
  Public Const TIME_ZONE_ID_UNKNOWN As Long = 0&
  Public Const TIME_ZONE_ID_STANDARD As Long = 1&
  Public Const TIME_ZONE_ID_DAYLIGHT As Long = 2&


  Public Declare Function GetTimeZoneInformation Lib "kernel32" (lpTimeZoneInformation As TIME_ZONE_INFORMATION) As Long
  Public Declare Sub CopyMemory Lib "kernel32" Alias "RtlMoveMemory" (lpDest As Any, lpSource As Any, ByVal cBytes&)

Sub Main()

  Dim tzi As TIME_ZONE_INFORMATION, sCurTimeName$, sMsg$

  ' call the function to get the timezone info.
  Select Case GetTimeZoneInformation(tzi)
    Case TIME_ZONE_ID_UNKNOWN
      sMsg = "Unknown timezone ID.  This error indicates that someone didn't set the ID correctly " _
              & "when using the SetTimeZoneInformation API function.  However, you may still be able " _
              & "to obtain information about the current timezone settings."


    Case TIME_ZONE_ID_STANDARD, TIME_ZONE_ID_DAYLIGHT
      ' the call was successful.  format the data for display....  if you wanted to only return the current
      ' timezone info, you would remove the second value from the line above (see comments below....)

      With tzi
        ' we have to copy the string data from the byte array in the structure (UDT) into a string variable.
        ' this is because the function returns uincode strings and VB automagically converts strings from
        ' unicode (the way VB stores strings internally) to ANSI strings and back again on any API call.
        ' thus, the data returned if you assign a string to the structure is not correct.  notice that the
        ' allocated space is 32 bytes and the data copied is 64 bytes.  this is because VB stores all
        ' strings internally as unicode strings (these are 2 bytes per character) so when you allocate a
        ' 32 byte string in VB, you are *actually* allocating 64 bytes of memory.  we use pointers to the
        ' two memory locations to copy the data from the buffer into the string variable.
        sCurTimeName = String$(32, vbNullChar)
        CopyMemory ByVal StrPtr(sCurTimeName), ByVal VarPtr(.StandardName(0)), 64&
  
        ' trim the string at the null terminator.
        sCurTimeName = StripNulls(sCurTimeName)
  
        ' format the date depending on how it was stored on this system.  there are two formats, absolute and
        ' Day-In-Month.  absolute stores the the exact year, month, day, hour, minute, etc... down to the
        ' millisecond of when the time change in the timezone takes place.  Day-In-Month format stores only the
        ' month, day of the week, week of the month and hour, minute, second, etc... for when the time change
        ' occures in this timezone.
        With .StandardDate
          If .wYear Then
            ' if the year member is non zero, the date is an absolute format date....
            sMsg = sCurTimeName & " Begins On " & CStr(.wMonth) & "/" & CStr(.wDay) & "/" & CStr(.wYear) & " At " & CStr(.wHour) & ":" & Format$(.wMinute, "00") & ":" & Format$(.wSecond, "00")
          Else
           ' otherwise it is a Day-In-Month format date....
            sMsg = sCurTimeName & " Begins On " & TranslateDay(.wDayOfWeek, .wDay) & " " & GetMonth(.wMonth) & " At " & .wHour & ":" & Format$(.wMinute, "00") & ":" & Format$(.wSecond, "00")
          End If
        End With
      End With
      
      ' continue formatting our message.
      sMsg = sMsg & vbCrLf & vbCrLf

    ' you would use the following line if you wanted to return only the current timezone info.
    ' you would also need to remove this value from the above "Case" statement.
    'Case TIME_ZONE_ID_DAYLIGHT

      ' format the dailight time portion of the return the same as we did for the standard time portion above....
      With tzi
        sCurTimeName = String$(32, vbNullChar)
        CopyMemory ByVal StrPtr(sCurTimeName), ByVal VarPtr(.DaylightName(0)), 64&
  
        sCurTimeName = StripNulls(sCurTimeName)
  
        With .DaylightDate
          If .wYear Then
            sMsg = sMsg & sCurTimeName & " Begins On " & CStr(.wMonth) & "/" & CStr(.wDay) & "/" & CStr(.wYear) & " At " & CStr(.wHour) & ":" & Format$(.wMinute, "00") & ":" & Format$(.wSecond, "00")
          Else
            sMsg = sMsg & sCurTimeName & " Begins On " & TranslateDay(.wDayOfWeek, .wDay) & " " & GetMonth(.wMonth) & " At " & .wHour & ":" & Format$(.wMinute, "00") & ":" & Format$(.wSecond, "00")
          End If
        End With
      End With


      ' add the bias information.  the bias is the difference, in minutes, between Coordinated Universal Time (UTC)
      ' and local time.  there are three bias members in a TIME_ZONE_INFORMATION structure.  the overall bias, the
      ' standard bias and the daylight bias.  these values are used when converting system time (this is UTC time)
      ' to local time and back again.
      sMsg = sMsg & vbCrLf & vbCrLf
      
      With tzi
        sMsg = sMsg & "Overall Bias:     " & CStr(.Bias / 60) & " Hours Difference Between Current Local Time" _
                                                  & " & Coordinated Universal Time (UTC)" & vbCrLf
        sMsg = sMsg & "Standard Bias:  " & CStr(.StandardBias / 60) & " Hours Difference Between Overall Bias" _
                                                  & " & Standard Time When Calculating Local To UTC Conversions" & vbCrLf
        sMsg = sMsg & "Daylight Bias   " & CStr(.DaylightBias / 60) & " Hours Difference Between Overall Bias" _
                                                  & " & Daylight Time When Calculating Local To UTC Conversions"
      End With


    Case TIME_ZONE_ID_INVALID
      sMsg = "Invalid timzone ID.  Unable to return timezone information."

  End Select
  
  
  
  ' display our message
  MsgBox sMsg, vbInformation, "GetTimeZoneInformation Demo"
  
  
End Sub

Private Function TranslateDay(ByVal nDayOfWeek&, ByVal nDay&) As String
  ' this function is used if the date formate is of the Day-In-Month type
  
  Dim sReturn$
  
  sReturn = "The "
  
  Select Case nDay
    Case 1: sReturn = sReturn & "First "
    Case 2: sReturn = sReturn & "Second "
    Case 3: sReturn = sReturn & "Third "
    Case 4: sReturn = sReturn & "Fourth "
    Case 5: sReturn = sReturn & "Last "
  End Select
  
  Select Case nDayOfWeek
    Case 0: sReturn = sReturn & "Sunday"
    Case 1: sReturn = sReturn & "Monday"
    Case 2: sReturn = sReturn & "Tuesday"
    Case 3: sReturn = sReturn & "Wednesday"
    Case 4: sReturn = sReturn & "Thursday"
    Case 5: sReturn = sReturn & "Friday"
    Case 6: sReturn = sReturn & "Saturday"
  End Select
  
  TranslateDay = sReturn & " In"
  
End Function

Private Function GetMonth(ByVal nMonth&) As String
  ' returns the name of the month
  
  Select Case nMonth
    Case 1: GetMonth = "January"
    Case 2: GetMonth = "February"
    Case 3: GetMonth = "March"
    Case 4: GetMonth = "April"
    Case 5: GetMonth = "May"
    Case 6: GetMonth = "June"
    Case 7: GetMonth = "July"
    Case 8: GetMonth = "August"
    Case 9: GetMonth = "September"
    Case 10: GetMonth = "October"
    Case 11: GetMonth = "November"
    Case 12: GetMonth = "December"
  End Select
End Function

Public Function StripNulls(ByVal sText As String) As String
' strips any nulls from the end of a string
  Dim nPosition&
  
  StripNulls = sText
  
  nPosition = InStr(sText, vbNullChar)
  If nPosition Then StripNulls = Left$(sText, nPosition - 1)
  If Len(sText) Then If Left$(sText, 1) = vbNullChar Then StripNulls = ""
End Function
